%AdelsonBergen.m

%% Define 'World' parameters
pWorld.n = [1,300,500];  %[ny, nx, nt] in pixels
pWorld.visAngle = 10;   %horizontal extent of visual angle
pWorld.dur = 1;        %duration (seconds)
pWorld.background = 0;

pWorld = initWorld(pWorld);  %get the x and y meshrid matrices

x = repmat(pWorld.x(1,:),pWorld.n(3),1);
t = repmat(pWorld.t',1,pWorld.n(2));

fontSize =12;


%% Space-time plot of a smoothly moving grating

v = 10;  %velocity (rightward degrees per second)
sf = 1;  %c/deg

%create the space-time image
movingGrating = cos(2*pi*sf*x-2*pi*v*t*sf);

%show the animation
figure(1)
animate(pWorld,movingGrating);

%Space-time plot
figure(2)
spaceTimeFFTPlot(pWorld,movingGrating,10,2,60);


%% Space-time plot of a sampled moving grating

v = 10;  %velocity (rightward degrees per second)
sf = 1;  %c/deg
frameRate = 30;  %frame rate for time steps
tStep = round(t*frameRate)/frameRate;

%create the space-time image
sampledGrating = cos(2*pi*sf*x-2*pi*v*tStep*sf);

%show the animation
figure(1)
animate(pWorld,sampledGrating);

%Space-time plot
figure(2)
spaceTimeFFTPlot(pWorld,sampledGrating,10,2,60);

%% Counterphase grating


v = 10;  %velocity (rightward degrees per second)
sf = 1;  %c/deg

%create the space-time image
counterphaseGrating = .5*cos(2*pi*sf*x-2*pi*v*t*sf)+.5*cos(2*pi*sf*x+2*pi*v*t*sf);

%show the animation
figure(1)
animate(pWorld,counterphaseGrating);

%Space-time plot
figure(2)
spaceTimeFFTPlot(pWorld,counterphaseGrating,10,2,60);



%% Space-time plot of a smoothly moving bar (Figure 3a a in paper)

v = 10;  %velocity (degrees per second)
v0 = -5;  %starting position;


%create the space-time image
ang = atan(v);
ramp = cos(ang)*(x-v0) - sin(ang)*t;
smoothBar = double(abs(ramp)<.05);

%show the animation
figure(1)
animate(pWorld,smoothBar);

%Space-time plot
figure(2)


spaceTimeFFTPlot(pWorld,smoothBar,10,2,60);

%%  space-time plot of a sampled moving bar (Figure 3c in paper)

v =10;  %velocity (degrees per second)
v0 = -5;  %starting position;
frameRate = 15;  %frame rate for time steps

tStep = round(t*frameRate)/frameRate;

%create the space-time image
ang = atan(v);

ramp = cos(ang)*(x-v0) - sin(ang)*tStep;
sampledBar = double(abs(ramp)<.05);

%show the animation
figure(1)
animate(pWorld,sampledBar);

%Space-time plot
figure(2)
spaceTimeFFTPlot(pWorld,sampledBar,10,2,60);


%% Difference between smooth motion and step motion (Figure 4c in paper)

figure(3)
clf

spaceTimeFFTPlot(pWorld,smoothBar-sampledBar,10,2,60);


%% Sinusoidal motion of an edge

sinEdge = x<0;

id = t>.05 & t<pWorld.dur-.05;
sinEdge(id) = pWorld.visAngle*sin(2*pi*(t(id)-.05)/(pWorld.dur-.1))/4-x(id) > 0;

sinEdge = double(sinEdge);

%show the animation over 2 seconds
figure(1)
animate(pWorld,sinEdge);

%Space-time plot
figure(2)
clf
spaceTimePlot(pWorld,sinEdge);


%% Sampled sinusoidal motion of an edge

frameRate = 30;  %frame rate for time steps
sampledSinEdge = x<0;

tStep = round(t*frameRate)/frameRate;

id = t>.05 & t<pWorld.dur-.05;
sampledSinEdge(id) = pWorld.visAngle*sin(2*pi*(tStep(id)-.05)/(pWorld.dur-.1))/4-x(id) > 0;

sampledSinEdge = double(sampledSinEdge);

%show the animation over 2 seconds
figure(1)
animate(pWorld,sampledSinEdge);

%Space-time plot
figure(2)
clf
spaceTimePlot(pWorld,sampledSinEdge);;
%% rightward moving random bar pattern

frameRate = 25;  %frame rate for time steps

nFrames = round(pWorld.dur*pWorld.n(3)/frameRate);

nx = 10;

strip = sign(randn(1,pWorld.n(2)/nx));
strip = repmat(strip,nx,1);
strip = strip(:)';

strip = repmat(strip,nFrames,1);

rightNoise = zeros(pWorld.n(3),pWorld.n(2));


start = 1;
while start<pWorld.n(3)
    rightNoise(start:(start+nFrames-1),:) = strip;
    strip = [strip(:,(end-nx+1):end),strip(:,1:(end-nx))];
    start = start+nFrames;
end

%show the animation
figure(1)
animate(pWorld,rightNoise);

%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,rightNoise,10,2,60);

%% rightward moving reverse phi

frameRate = 25;  %frame rate for time steps

nFrames = round(pWorld.dur*pWorld.n(3)/frameRate);

nx = 10;

strip = sign(randn(1,pWorld.n(2)/nx));
strip = repmat(strip,nx,1);
strip = strip(:)';

strip = repmat(strip,nFrames,1);

reversePhi = zeros(pWorld.n(3),pWorld.n(2));


start = 1;
while start<pWorld.n(3)
    reversePhi(start:(start+nFrames-1),:) = strip;
    strip = -[strip(:,(end-nx+1):end),strip(:,1:(end-nx))];
    start = start+nFrames;
end

%show the animation
figure(1)
animate(pWorld,reversePhi);

%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,reversePhi,10,2,60);

%%  Rightward-moving square wave

frameRate = 20;
nFrames = round(pWorld.dur*pWorld.n(3)/frameRate);
nCycles = 5;

nx= .25*pWorld.n(2)/nCycles;

strip = sign(sin(nCycles*linspace(0,pi*2,pWorld.n(2))));


strip = repmat(strip,nFrames,1);

rightSquare = zeros(pWorld.n(3),pWorld.n(2));


start = 1;
while start<pWorld.n(3)
    rightSquare(start:(start+nFrames-1),:) = strip;
    strip = [strip(:,(end-nx+1):end),strip(:,1:(end-nx))];
    start = start+nFrames;
end
rightSquare = rightSquare(1:pWorld.n(3),:);

%show the animation
figure(1)
animate(pWorld,rightSquare);

%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,rightSquare,10,2,60);

%%   Rightward-moving 'fluted' square wave


frameRate = 20;
nFrames = round(pWorld.dur*pWorld.n(3)/frameRate);
nCycles = 5;

nx= .25*pWorld.n(2)/nCycles;

strip = sign(sin(nCycles*linspace(0,pi*2,pWorld.n(2))));
strip = strip - 4/pi*sin(nCycles*linspace(0,pi*2,pWorld.n(2)));

strip = repmat(strip,nFrames,1);

rightFlute = zeros(pWorld.n(3),pWorld.n(2));


start = 1;
while start<pWorld.n(3)
    rightFlute(start:(start+nFrames-1),:) = strip;
    strip = [strip(:,(end-nx+1):end),strip(:,1:(end-nx))];
    start = start+nFrames;
end
rightSquare = rightSquare(1:pWorld.n(3),:);

%show the animation
figure(1)
animate(pWorld,rightFlute);

%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,rightFlute,10,2,60);

%% First order motion (moving Gabor)


v = 10;  %velocity (rightward degrees per second)
v0 = -5;  %starting position;
sf = 1;  %c/deg
sig  =.35;
%create the space-time image
ang = atan(v);

ramp = cos(ang)*(x-v0) - sin(ang)*t;

movingGrating = cos(2*pi*sf*x-2*pi*v*t*sf);
firstOrderMotion = movingGrating.*(exp(-abs(ramp)/sig.^2));

%show the animation
figure(1)
animate(pWorld,firstOrderMotion);


%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,firstOrderMotion,10,2,60);

%% Second order motion


v = 10;  %velocity (degrees per second)
v0 = -5;  %starting position;
sf =1;
sig = .35;

%create the space-time image
ang = atan(v);
ramp = cos(ang)*(x-v0) - sin(ang)*t;


%stationaryGrating = cos(2*pi*sf*x);
%stationaryGrating  = randn(pWorld.n(3),pWorld.n(2));
stationaryGrating = repmat(randn(1,pWorld.n(2)),pWorld.n(3),1);
secondOrderMotion = stationaryGrating.*(exp(-abs(ramp)/sig.^2));

%show the animation
figure(1)
animate(pWorld,secondOrderMotion);


%Space-time plot
figure(2)
clf
spaceTimeFFTPlot(pWorld,secondOrderMotion,10,2,60);


%% Make four space-time separable filters

%2x2 with two phases and two time constants
clear pRF separableRF

phases = [0,-90];                           %two phases
k = 1.33;
thetas = {[10,10*k]/1000,[13,13*k]/1000};   %two time constants
delays = [.01,.015];
figure(4)
clf
count =1;
for phaseNum = 1:2
    for thetaNum = 1:2
        pRF.center = [0,0];
        pRF.sf = 1;  %c/deg
        pRF.sig = .4; %
        pRF.ang = 0; %orientation (deg)
        pRF.phase = phases(phaseNum); %0 = sin phase
        pRF.contrast = 1;

        pRF.n = [3,4]; %n cascades
        pRF.theta = thetas{thetaNum}; %sec
        pRF.a = [.03,.03];
        pRF.delay = delays(thetaNum);


        pRF.nt = 100;
        pRF.nw = 100;

        tmp = makeSeparableSTFilter(pWorld,pRF(1));
        tmp = squeeze(tmp)';
        tmp = tmp(:,((pWorld.n(2)/2-pRF.nw/2)+1):(pWorld.n(2)/2+pRF.nw/2));
        separableRF{count} =tmp;

        subplot(1,4,count);
        %plot the separable RF (see figure 10c)
        [xx,tt] = spaceTimePlot(pWorld,separableRF{count});
        count = count+1;
    end
end

%plot the cross-sections of the filters (See figures 10a and b in the paper)
figure(5)
clf
subplot(2,1,1)
plot(xx,separableRF{1}(30,:),'r-');

hold on
plot(xx,separableRF{3}(30,:),'g-');
xlabel('Space (deg)','FontSize',fontSize);
set(gca,'FontSize',fontSize);
set(gca,'Color','k');

subplot(2,1,2)
plot(1000*tt,separableRF{3}(:,end/2),'y-');
hold on
plot(1000*tt,separableRF{4}(:,end/2),'b-');

xlabel('Time (msec)','FontSize',fontSize);
set(gca,'Color','k');

%% Construct spatiotemporally oriented filters from separable pairs

%see figure 10c in the paper

orientedRF{1} = separableRF{4} + separableRF{1};  %leftward (sin)
orientedRF{2} = separableRF{3} - separableRF{2};  %leftward (cos)
orientedRF{3} = separableRF{3} + separableRF{2};  %rightward (sin)
orientedRF{4} = separableRF{4} - separableRF{1};  %rightward (cos)

figure(6)
clf
for i=1:4
    subplot(1,4,i)
    [xx,tt] = spaceTimePlot(pWorld,orientedRF{i});
end


figure(3)
[fampL,wx,wt]= spaceTimeFFTPlot(pWorld,orientedRF{1},10,3,60);
[fampR,wx,wt]= spaceTimeFFTPlot(pWorld,orientedRF{3},10,3,60);


%%  Now, calculate the response to these filters to some of the stimuli.

%see figure 15 in the paper

%stim = movingGrating;
stim = counterphaseGrating;
%stim = sampledGrating;


%stim = smoothBar;
%stim = sampledBar;

%stim = sinEdge;
%stim =  sampledSinEdge;

%stim = rightNoise;
%stim = reversePhi;

%stim = rightSquare;
%stim = rightFlute;

%stim = firstOrderMotion;
%stim = secondOrderMotion;


%linear response each unit:

%Let's add a ramp to the beginning and end to remove transients

rampFrames = 50;  %frames

ramp = repmat(linspace(0,1,rampFrames)',1,pWorld.n(2));
stim(1:rampFrames,:) = stim(1:rampFrames,:).*ramp;
stim((end-rampFrames+1):end,:) =stim((end-rampFrames+1):end,:).*flipud(ramp);

dx = x(1,2)-x(1,1);
dt = t(2,1)-t(1,1);
for i=1:4
    tmp = conv2(stim,orientedRF{i},'same');
    tmp = tmp*dt;
    resp{i} = tmp;
end

leftEnergy = (resp{1}.^2+resp{2}.^2);
rightEnergy = (resp{3}.^2+resp{4}.^2);

% Motion opponency
sig = .01;
opponentMechanism = (rightEnergy-leftEnergy)./(leftEnergy+rightEnergy+sig);

%scFac = 1/max([max(leftEnergy(:)),max(rightEnergy(:))]);

scFac = 30;
figure(8)
clf

subplot(2,3,1);
spaceTimePlot(pWorld,scFac*leftEnergy);
title('Left');

subplot(2,3,2)
spaceTimePlot(pWorld,scFac*rightEnergy);
title('Right');

subplot(2,3,3)
spaceTimePlot(pWorld,opponentMechanism);
title('Right-Left');

%plot time-course for an RF centered in the image

maxy = ceil(max(max(leftEnergy(:),rightEnergy(:)))*100)/100;

subplot(2,3,4)
plot(pWorld.t*1000,leftEnergy(:,end/2));
set(gca,'XLim',[0,pWorld.dur*1000]);
set(gca,'YLim',[0,maxy]);
xlabel('Time (msec)','FontSize',fontSize);
ylabel('Response in center','FontSize',fontSize);

subplot(2,3,5)
plot(pWorld.t*1000,rightEnergy(:,end/2));
set(gca,'XLim',[0,pWorld.dur*1000]);
set(gca,'YLim',[0,maxy]);
xlabel('Time (msec)','FontSize',fontSize);


subplot(2,3,6)

plot([0,pWorld.dur*1000],[0,0],'r:');
hold on
plot(pWorld.t*1000,opponentMechanism(:,end/2));
set(gca,'XLim',[0,pWorld.dur*1000]);
xlabel('Time (msec)','FontSize',fontSize);
set(gca,'YLim',[-1,1]);

figure(2)
clf
spaceTimeFFTPlot(pWorld,stim,10,2,60);
subplot(1,2,2)
%draw the contour plot of the RF on top of the FFT
hold on
contour(wx,wt,fampL,.5,'r-');
contour(wx,wt,fampR,.5,'g-');

%% a little figure about second order motion


figure(9)
clf
hold on
id = 1:50:pWorld.n(2);

for i=1:length(id)
    stairs(pWorld.x(1,:),secondOrderMotion(id(i),:)'+i*2);
end
xlabel('Space (deg)');
set(gca,'YDir','reverse')
set(gca,'Color','w');
set(gca,'YTick',[-inf,inf]);




