%EnergyModelDemo.m

%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [50,50,1001];   %[ny, nx, nt] in pixels
pWorld.visAngle = 4;       %horizontal extent of visual angle
pWorld.dur = 1;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% Define the four 'quadrature' V1 cells

clear pRF

%spatial parameters
pRF.center = [0,0];
pRF.sf =  1;  %c/deg
pRF.sig = .5; %
pRF.ang = 90; %orientation (deg)
pRF.phase = 0; %0 = sin phase
pRF.contrast = 1;

%temporal parameters
pRF.n = [3,5]; %n cascades
pRF.theta = [.005,.005]; %sec
pRF.a = [.025,0];
pRF.nt = 50; % frames
phaseList = [0,90,180,270];


clear RF
for i=1:4
    pRF.phase = phaseList(i);
    RF{i} = makeSeparableSTFilter(pWorld,pRF);

    %show the whole receptive field (as a movie)
    figure(1)
    showRF(pWorld,RF{i})

    %show the space-time plot by slicing through one plane in space
    nn = ceil(pWorld.n(2)/2);  %half the size of the second (x) dimension
    figure(2)
    showRF(pWorld,squeeze(RF{i}(:,nn,:)));
end


%% Stimulus: drifting grating

pStim.type = 'DriftingGrating';
pStim.sf = 1;
pStim.tf = 4; %Hz
pStim.ang = 90;
pStim.phase = 90;
pStim.contrast = .32;
pStim.p0 = [0,0];
stim = makeStim(pWorld,pStim); %horizontally oriented grating

%set beginning and end to zero contrast

stim(:,:,pWorld.t<.15 | pWorld.t>.85) = 0;


%figure(3)
%showStimulus(pWorld,stim,RF{1}(:,:,20));

%% Linear response for each V1 neuron

L = zeros(length(RF),pWorld.n(3));
for i=1:4
    L(i,:) = linearResponse(pWorld,stim,RF{i});
end

figure(3)
clf
subplot(3,1,1)
plot(pWorld.t,L');
title('Linear response','FontSize',fontSize);

%% Rectify and square

clear pModel
pModel.k = .1;  %scale factor
pModel.Vrest = 0;     %threshold
pModel.p = 2;         %exponent

A = powerLawModel(pModel,L);

subplot(3,1,2)
plot(pWorld.t,A);

title('Half squared response','FontSize',fontSize);
%% 'Energy' mechanism is sum of responses to linear filters

E = mean(A);

subplot(3,1,3)
plot(pWorld.t,E);
xlabel('Time (s)','FontSize',fontSize);
title('''Energy'' output','FontSize',fontSize);

%% Moving bar stimulus

clear pStim
pStim.type = 'bar';
pStim.ang =90;
pStim.length = 20;
pStim.width = .25;
pStim.contrast = 1;
pStim.dir = 0;
pStim.p0 = [0,-pWorld.visAngle/2];
pStim.v = -pWorld.visAngle;  %deg/sec

stim = makeStim(pWorld,pStim);

figure(4)
showStimulus(pWorld,stim)


L = zeros(length(RF),pWorld.n(3));
for i=1:4
    L(i,:) = linearResponse(pWorld,stim,RF{i});
end

figure(5)
clf
subplot(3,1,1)
plot(pWorld.t,L');
title('Linear response','FontSize',fontSize);

%% Rectify and square

clear pModel
pModel.k = .1;  %scale factor
pModel.Vrest = 0;     %threshold
pModel.p = 2;         %exponent

A = powerLawModel(pModel,L);

subplot(3,1,2)
plot(pWorld.t,A);

title('Half squared response','FontSize',fontSize);
%% 'Energy' mechanism is sum of responses to linear filters

E = mean(A);

subplot(3,1,3)
plot(pWorld.t,E);
xlabel('Time (s)','FontSize',fontSize);
title('''Energy'' output','FontSize',fontSize);

%% 'Tile' spatial frequencies and orientations with filters

sfList = [.5,1,2];  %c/deg
nsf = length(sfList);
%sfList = pStim.sf;
angList = 0:45:179;
nang = length(angList);


clear RF
for i=1:length(sfList)
    pRF.sf = sfList(i);
    pRF.sig = .5/pRF.sf;
    for j=1:length(angList)
        pRF.ang = angList(j);
        for k=1:4  %phases
            pRF.phase = phaseList(k);
            RF{i,j,k} = makeSeparableSTFilter(pWorld,pRF);
        end
    end
end

%% show the RFs

x = 0;
count  =1;
figure(6)
for i=1:nsf
    for j=1:nang
        subplot(nsf,nang,count)
        image((RF{i,j,1}(:,:,10)+1)*127.5)
        axis equal
        axis off
        count = count+1;
    end
end

cmap = zeros(256,3);
cmap(1:128,1) = linspace(1,0,128).^.6;
cmap(129:256,2) = linspace(0,1,128);
colormap(cmap);

