%ModelFest.m

%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [256,256,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 2.133;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices


%% load in the ModelFest thresholds and images

%subjectNames modelFestThresh modelFestSEM
load ModelFestThresholds

%ModelFestStimuli stim stimName
load ModelFestStimuli stim stimName

img = reshape(stim,256,256*43);

figure(1)
clf
subplot('Position',[.05,.9,.9,.9/43])
image(img);
colormap(gray(256))
axis off


subplot('Position',[.05,.05,.9,.85])

plot(-log(thresh),'.','MarkerSize',20)

set(gca,'XLim',[.5,43.5]);
hold on

ylim = get(gca,'YLim');
for i=.5:44
    plot(i*[1,1],ylim,'-','Color',[.8,.8,.8]);
end
logy2raw
xlabel('Stimulus number','FontSize',fontSize);
ylabel('1/threshold','FontSize',fontSize);

%% Bank of filters

clear pRF RF

sfList = [4];        %list of preferred spatial frequencies
ctList = [1];           %list of scale factors for these sf filters
angList = 0:45:179;         %list of preferred orientations
phaseList = [0,90,180,270]; %Quadrature phases

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf = NaN;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = NaN;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = NaN;              %orientation (deg) (to be varied)
pRF.phase = 0;              %0 = sin phase
pRF.contrast = NaN;           %scale factor (to be varied with sf)
pRF.nt = 1;                 %number of temporal frames


sigFac = .2;
%Generate a cell array containing the receptive fields by calling 'makeGaborRF'
%Columns (1st dimension) will be orientation
%Rows (2nd dimension0 will be spatial frequency
%Third dimension are the four phases (Quadrature phase)
for i=1:length(sfList)
    pRF.sf = sfList(i);         %spatial frequency
    pRF.sig = sigFac/pRF.sf;       %width is inversely proportional to sf
    pRF.contrast = ctList(i);   %contrast
    for j=1:length(angList)
        pRF.ang = angList(j);   %orientation
        for k=1:4  %quadrature phases
            pRF.phase = phaseList(k);
            RF{i,j,k} = makeGaborRF(pWorld,pRF);
        end
    end
end

%% Model Parameters

%Half wave rectification and nonlinearity parameters
pModel.k = 1;         %scale factor
pModel.Vrest = 0;       %threshold
pModel.p = 2;           %exponent

%Normalization parameters
pModel.sig = .1;        %semisaturation constant
pModel.a = 1;           %scale factor

%% Calculate the model response to each stimulus\
clear C
p = 8;
% for i=1:size(stim,3)
%     C = normalizationModel(pWorld,pModel,stim(:,:,i),RF);
%     R(i) = (sum(C(:).^p))^(1/p);
% end

%linear model
for i=1:size(stim,3)
    for j=1:size(RF,1)
        for k=1:size(RF,2)
            for l = 1:size(RF,3)
               Vm = linearResponse(pWorld,stim(:,:,i),RF{j,k,l});
                C(j,k,l) = powerLawModel(pModel,Vm);
            end
        end
    end
   
    R(i) = (sum(C(:).^p))^(1/p);
end


figure(1)
hold on

a = R;
b = mean(thresh,2);

scFac = mean(b)/mean(a);
plot(-log(R*scFac),'ko-');

%% scatterplot

figure(2)

plot(-log(b),-log(R*scFac),'o','MarkerFaceColor','b');
xlabel('Psychophysical thresholds');
ylabel('Model response');

z = corrcoef(-log(b),-log(a*scFac));
title(sprintf('r= %5.2f',z(1,2)));
logx2raw
logy2raw

%% show the filters

figure(3)
clf
count  =1;
for i=1:length(sfList)
    for j=1:length(angList)
        subplot(length(sfList),length(angList),count)
        h=image((RF{i,j,1}+1)*127.5)
        axis equal
        axis off
        count = count+1;
    end
end

cmap = zeros(256,3);
cmap(1:128,1) = linspace(1,0,128).^.6;
cmap(129:256,2) = linspace(0,1,128);
colormap(cmap);
