%RustMT.m

clear all
%define model parameters
p.pList = 30:30:360;  %list of each V1 neuron's preferred directions
s.dList = 30:30:360;  %list of possible stimulus directions

p.b = 180*pi/180;   %tuning width of V1 neurons
% p.s1 = .1;          %'untuned' normalization factor
% p.s2 = .1;          %'self' normalization factor
p.s1 = .1;          %'untuned' normalization factor
p.s2 = .1;          %'self' normalization factor

p.A = 1;            %MT scaling nonlinearity
p.B = 2;            %MT exponent nonlinearity


%make up some weights of V1 influence on MT response:
%use this line for pattern cell
%p.w = exp(-(linspace(-1,1,length(p.pList)).^2)/.4)-.3;

%use this line for a component cell
p.w = exp(-(linspace(-1,1,length(p.pList)).^2)/.01); 
p.w(3) = .2;
%Plot the linear weights of V1 neurons
figure(1);
clf
plot(p.pList,p.w,'bo-','MarkerFaceColor','b');
xlabel('Preferred orientation (deg)');
ylabel('Weight');
xlabel('Direction (deg)');
set(gca,'XTick',0:45:360);
set(gca,'XLim',[0,360]);

%Now we will make the stimuli used to generate the matrix, M, which is the 
%response to each pairwise combination of gratings that make all possible 
%plaids (as in figure 4).
count = 0;
s.c = zeros(length(s.dList)^2,length(s.dList));
for i=1:length(s.dList)
    for j=1:length(s.dList);
        count = count+1;
        s.c(count,i) = s.c(count,i)+1/6;
        s.c(count,j) = s.c(count,j)+1/6;
    end
end

%get the model responses
M = predRustMT(p,s);
%reshape it into a square
M = reshape(M,length(s.dList),length(s.dList));

%Display the matrix like they do in figure 4
figure(2);
clf
imagesc(s.dList,s.dList,M);
colormap(gray)
hold on
contour(s.dList,s.dList,M,8,'-','LineWidth',2,'Color',[1,.5,.5]);
set(gca,'YDir','normal');
axis equal
axis tight
xlabel('Direction 1 (deg)');
ylabel('Direction 2 (deg)');
set(gca,'XTick',0:90:360);
set(gca,'YTick',0:90:360);

figure(3)
clf
surf(s.dList,s.dList,M);
set(gca,'XLim',[0,360]);
set(gca,'YLim',[0,360]);
set(gca,'XTick',0:90:360);
set(gca,'YTick',0:90:360);


%% Response to gratings vs. 120 deg plaids

%Single gratings
s.c= zeros(length(s.dList),length(s.dList));
for i=1:length(s.dList)
    s.c(i,i) = 1/6;
end

M = predRustMT(p,s);
figure(4)
clf
h1=plot(s.dList,M,'r-o','MarkerFaceColor','r');
hold on


%Component prediction (linear sum of shifted responses to single gratings)

CP=M([3:12 1:2]) + M([11:12 1:10]);
h2= plot(s.dList,CP,'go-','MarkerFaceColor','g');





%response 120 deg plaids
s.c= zeros(length(s.dList),length(s.dList));
for i=1:length(s.dList)
    s.c(i,mod(i-3,length(s.dList))+1) = 1/6;
    s.c(i,mod(i+1,length(s.dList))+1) = 1/6;
end

M = predRustMT(p,s);
h3= plot(s.dList,M,'bo-','MarkerFaceColor','b')

legend([h1,h2,h3],{'Single grating','Predicted component response','Model response'});
xlabel('Direction (deg)');
set(gca,'XTick',0:45:360);
set(gca,'XLim',[0,360]);
ylim = get(gca,'YLim');
set(gca,'YLim',[0,ylim(2)]);

ylabel('Response');


