%SpatialFrequencyAdaptation.m


%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [100,100,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 1;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices


%% Model Parameters

%Half wave rectification and nonlinearity parameters
pModel.k = 500;         %scale factor
pModel.Vrest = 0;       %threshold
pModel.p = 2;           %exponent

%Normalization parameters
pModel.sig = .1;        %semisaturation constant
pModel.a = 1;           %scale factor

pModel.quick = 6;       %quick pooling factor

%% Create a population of spatio-temporal frequency filters
clear pRF RF

sfList = exp(linspace(log(2),log(24),11));        %list of preferred spatial frequencies
%sfList = 9;

filterSF = sfList;
%ctList = ones(size(sfList));
ctList = .1*sfList.^2;
%ctList = 1./sqrt(sfList);          %list of scale factors for these sf filters
angList = 0:45:179;         %list of preferred orientations
phaseList = [0,90,180,270]; %Quadrature phases

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf = NaN;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = NaN;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = NaN;              %orientation (deg) (to be varied)
pRF.phase = 0;              %0 = sin phase
pRF.contrast = NaN;           %scale factor (to be varied with sf)
pRF.nt = 1;                 %number of temporal frames

%Generate a cell array containing the receptive fields by calling 'makeGaborRF'
%Columns (1st dimension) will be orientation
%Rows (2nd dimension0 will be spatial frequency
%Third dimension are the four phases (Quadrature phase)
clear RF
for i=1:length(sfList)
    pRF.sf = sfList(i);         %spatial frequency
    pRF.sig = .5/pRF.sf;       %width is inversely proportional to sf
    pRF.contrast = ctList(i);   %contrast
    for j=1:length(angList)
        pRF.ang = angList(j);   %orientation
        for k=1:4  %quadrature phases
            pRF.phase = phaseList(k);
            RF{i,j,k} = makeGaborRF(pWorld,pRF);
        end
    end
end

%% show the RFs

figure(1)
clf
count  =1;
for i=1:length(sfList)
    for j=1:length(angList)
        subplot(length(sfList),length(angList),count)

        h=image((RF{i,j,1}+1)*127.5);
        axis equal
        axis off
        count = count+1;
    end
end

cmap = zeros(256,3);
cmap(1:128,1) = linspace(1,0,128).^.6;
cmap(129:256,2) = linspace(0,1,128);
colormap(cmap);


%% Spatial Frequency response function


sfList = exp(linspace(log(1),log(50),31));

%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = NaN;                       %spatial frequency (c/deg)
pStim.ang = 90;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = .01;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center

resp = zeros(size(sfList));
respAll = zeros(length(filterSF),length(sfList));
for i=1:length(sfList)
    pStim.sf = sfList(i);
    stim = makeStim(pWorld,pStim);  %horizontally oriented grating

    resp(i) = normalizationModel(pWorld,pModel,stim,RF,'quick',pModel.quick);
    tmp = normalizationModel(pWorld,pModel,stim,RF);
    respAll(:,i) = tmp(:,3);
end

figure(2)
clf
plot(log(sfList),resp,'b-');
set(gca,'XTick',log([1,2,4,8,16,32,64]));
hold on
plot(log(sfList),respAll');
logx2raw(exp(1),0)
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);


%% Spatial frequency contrast threshold function


sfList = exp(linspace(log(1),log(50),31));

%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = NaN;                       %spatial frequency (c/deg)
pStim.ang = 90;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = NaN;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center



respThresh = .001;

lo = zeros(size(sfList));
hi = ones(size(sfList));


%Binary search
for j=1:21
    ctList = (hi+lo)/2;
    resp = zeros(size(sfList));
    for i=1:length(sfList)
        pStim.sf = sfList(i);
        pStim.contrast = ctList(i);
        stim = makeStim(pWorld,pStim);  %horizontally oriented grating

        resp(i) = normalizationModel(pWorld,pModel,stim,RF,'quick',pModel.quick);
    end
    lo(resp<respThresh) = ctList(resp<respThresh);
    hi(resp>respThresh) = ctList(resp>respThresh);
    j
end

thresh = (hi+lo)/2;
thresh(resp<respThresh*.95) = NaN;

figure(3)
clf
plot(log(sfList),-log(thresh),'b-');
set(gca,'XTick',log([1,2,4,8,16,32,64]));

logx2raw(exp(1),0)
logy2raw(exp(1),1);
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('1/contrast threshold','FontSize',fontSize);




%% Now measure the response to an adapting stimulus.


adaptSF =9;
%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = adaptSF;                       %spatial frequency (c/deg)
pStim.ang = 90;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = 1/3;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center
stim = makeStim(pWorld,pStim);  %horizontally oriented grating


%uncomment this line to make a square wave!
%stim = sign(stim);
adaptResp = normalizationModel(pWorld,pModel,stim,RF);

adaptFac = .1;

%Scale the RF's based on the response to the stimulus.

for i=1:length(filterSF)
    for j=1:4
        for k=1:4
            RF{i,j,k} = RF{i,j,k}/(1+adaptFac*adaptResp(i,j));
        end
    end
end

%% Measure the thresholds again

% Spatial Frequency response function


sfList = exp(linspace(log(1),log(50),31));

%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = NaN;                       %spatial frequency (c/deg)
pStim.ang = 90;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = NaN;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center



respThresh = .001;

lo = zeros(size(sfList));
hi = ones(size(sfList));


%Binary search
for j=1:21
    ctList = (hi+lo)/2;
    resp = zeros(size(sfList));
    for i=1:length(sfList)
        pStim.sf = sfList(i);
        pStim.contrast = ctList(i);
        stim = makeStim(pWorld,pStim);  %horizontally oriented grating

        resp(i) = normalizationModel(pWorld,pModel,stim,RF,'quick',pModel.quick);


    end
    lo(resp<respThresh) = ctList(resp<respThresh);
    hi(resp>respThresh) = ctList(resp>respThresh);
    j
end

adaptThresh = (hi+lo)/2;
adaptThresh(adaptThresh<respThresh*.95) = NaN;

figure(3)
hold on
plot(log(sfList),-log(adaptThresh),'bs-','MarkerFaceColor','b');
set(gca,'XTick',log([1,2,4,8,16,32,64]));

logx2raw(exp(1),0)
logy2raw(exp(1),1);


%% show differences

figure(4)
clf
hold on
%plot(log(sfList),-log(thresh)+log(adaptThresh),'go-','MarkerFaceColor','g');

plot(log(sfList),adaptThresh./thresh,'bo-');
set(gca,'XTick',log([1,2,4,8,16,32,64]));
ylim = get(gca,'YLim');
plot(log(adaptSF)*[1,1],ylim,'r:');
logx2raw(exp(1),0)
xlabel('Spatial frequency (c/deg)','FontSize',fontSize);
ylabel('Relative threshold deviation','FontSize',fontSize);
%logy2raw(exp(1),1);




