%SpatioTemporalEnvelope.m
%
%Matlab code to generate the classic contrast sensitivity figure in which
%spatial frequency increases (exponentially) from left to right.
%Contrast increases from top to bottom. The boundary between the invisible and
%visible gratings is shaped like the human CSF (contrast sensitivity function).
%
%Note, however, that the apparent contrast of the bottom of the image (high
%contrast) appears roughly equal across spatial frequencies - a
%demonstration of 'contrast constancy'.
%
%Written by G.M. Boynton, August 2005

sz = [800,800];  %size of image [y,x], in pixels

[x,y]= meshgrid(linspace(1,5,sz(2)),linspace(0,1,sz(1)));

img = sin(exp(x)).*(y.^3);


figure(1)
clf
h = image((img+1)*128);
colormap(gray(256));
axis off

tf = 4;  %Hz

frameRate = 40; %Hz
t = 0;
tic

ramp = repmat(linspace(-.5,.5,256)',1,3);

slider = uicontrol('Units','normalized','Style','Slider','Position',[0,0,1,.05],'Min',0,'Max',16);
txt = uicontrol('Units','normalized','Style','Slider','Position',[0,.05,.1,.05],'Style','text');
stop = uicontrol('Units','normalized','Style','togglebutton','Position',[0,.95,.1,.05],'String','Stop','Value',0);
go = uicontrol('Units','normalized','Style','togglebutton','Position',[.1,.95,.1,.05],'String','Go','CallBack','SpatioTemporalEnvelope');

while ~get(stop,'Value')
    tf = get(slider,'Value');
    set(txt,'String',sprintf('%5.1f Hz',tf));
    ct = cos(2*pi*t*tf);
    %set(h,'CData',(ct*img+1)*128);
    
    
    cmap = (ct*ramp+.5);
    colormap(cmap);
    
    drawnow
    t = t+1/frameRate;
    while toc<t
    end
end
set(stop,'Value',0);



