%SpikeTriggeredAverageDemo.m


%% Define 'World' parameters
pWorld.n = [17,17,2001];  %[ny, nx, nt] in pixels
pWorld.visAngle = 10;   %horizontal extent of visual angle
pWorld.dur = (pWorld.n(3)-1)/1000;        %duration (seconds)
pWorld.background = 0;

pWorld = initWorld(pWorld);  %get the x and y meshrid matrices

fontSize =12;


%% Simple cell RF (Gabor)
clear pRF
pRF.center = [0,0];
pRF.sf = .25;  %c/deg
pRF.sig = 1.75; %
pRF.ang = 90; %orientation (deg)
pRF.phase = -90; %0 = sin phase
pRF.contrast = 1;

pRF.n = [3,5]; %n cascades
pRF.theta = [.006,.006]; %sec
pRF.a = [.035,.03];
pRF.nt = 100;

RF = makeSeparableSTFilter(pWorld,pRF);


figure(1)
clf
showRF(pWorld,RF);

figure(2)
clf
showRF(pWorld,RF(:,:,17));

figure(3)
showRF(pWorld,squeeze(RF(:,ceil(pWorld.n(1)/2),:)));

%% Stimulus

% %impulse grating
% pStim.phase = 0;
% stim = zeros(pWorld.n);
%
% stim(2,2,1:10) = 1;

%random stimulus
pStim.frame = 10;  %msec

stim = zeros(pWorld.n);
for i=1:pStim.frame:(pWorld.n(3)-pStim.frame)
    img = 2*(rand(pWorld.n(1:2))-.5);
    stim(:,:,i:(i+pStim.frame-1))=repmat(img,[1,1,pStim.frame]);
end

figure(1)
clf
showStimulus(pWorld,stim,RF(:,:,20));


%% linear model
Vm = linearResponse(pWorld,stim,RF);

figure(4)
clf
subplot(3,1,1)
plot(pWorld.t,Vm);
xlabel('Time (sec)','FontSize',fontSize);
ylabel('Resp','FontSize',fontSize);



%% Define and plot Power law model converting membrane potential to spike rate
clear pModel
pModel.k = .1;
pModel.Vrest = 0;
pModel.p = 2;

spikeRate = powerLawModel(pModel,Vm);

subplot(3,1,2)
plot(pWorld.t,spikeRate,'b-');
xlabel('Time (sec)','FontSize',fontSize);
ylabel('Firing Rate (spikes/sec)','FontSize',fontSize);


%% Convert spike rate to spike train

spikes = spikeRate2Spikes(pWorld,spikeRate);

subplot(3,1,3)
stairs(pWorld.t,spikes);
sound(spikes,1/pWorld.t(2))

%% Spike triggered average
binWidth = 20;
STA = getSTA(pWorld,spikes,stim,size(RF,3));

figure(5)
showRF(pWorld,STA/max(STA(:)));
set(5,'Position',[100    34   200   200]);
drawnow
figure(6)
set(6,'Position',[101   247   200   200]);
showRF(pWorld,STA(:,:,12)/max(STA(:)));
drawnow

figure(7)
showRF(pWorld,squeeze(STA(:,ceil(pWorld.n(1)/2),:)/max(STA(:))));
set(7,'Position',[100   456   200   200]);
figure(8)

cmap = zeros(256,3);
cmap(1:128,1) = linspace(1,0,128).^.6;
cmap(129:256,2) = linspace(0,1,128);
colormap(cmap);
clf
set(gcf,'Color',[.25,.25,.25]);
subplot(1,2,1)
contour(RF(:,:,12)/max(RF(:)),linspace(-1,1,20));
set(gca,'YDir','reverse')
axis equal
axis off




%% repeat this n times

nReps = 500;
pStim.frame = 10;  %msec


STA = zeros(pWorld.n(1),pWorld.n(2),size(RF,3));

for repNum=1:nReps

    %random stimulus
    stim = zeros(pWorld.n);
    for i=1:pStim.frame:(pWorld.n(3)-pStim.frame)
        %img = 2*(rand(pWorld.n(1:2))-.5);
        %img = randn(pWorld.n(1:2));
        img = sign(rand(pWorld.n(1:2))-.5);
        stim(:,:,i:(i+pStim.frame-1))=repmat(img,[1,1,pStim.frame]);
    end

    %linear response
    Vm = linearResponse(pWorld,stim,RF);
    spikeRate = powerLawModel(pModel,Vm);
    spikes = spikeRate2Spikes(pWorld,spikeRate);
    STA = STA+ getSTA(pWorld,spikes,stim,size(RF,3))/nReps;
    if mod(repNum,10) == 0
        repNum

        figure(5)
        showRF(pWorld,STA/max(STA(:)));
        set(5,'Position',[100    34   200   200]);
        drawnow
        figure(6)
        set(6,'Position',[101   247   200   200]);
        showRF(pWorld,STA(:,:,12)/max(STA(:)));
        drawnow

        figure(7)
        showRF(pWorld,squeeze(STA(:,ceil(pWorld.n(1)/2),:)/max(STA(:))));
        set(7,'Position',[100   456   200   200]);
        figure(8)

        cmap = zeros(256,3);
        cmap(1:128,1) = linspace(1,0,128).^.6;
        cmap(129:256,2) = linspace(0,1,128);
        colormap(cmap);
        clf
        set(gcf,'Color',[.25,.25,.25]);
        subplot(1,2,1)
        contour(RF(:,:,12)/max(RF(:)),linspace(-1,1,20));
        set(gca,'YDir','reverse')
        axis equal
        axis off

        subplot(1,2,2)
        contour(STA(:,:,12)/max(STA(:)),linspace(-1,1,20));
        set(gca,'YDir','reverse')
        axis equal
        axis off
        set(8,'Position',[ 317   127   560   420]);
        drawnow

    end
end
