function RF = makeDOG1RF(pWorld,pRF)
%RF = makeDOG1RF(pWorld,pRF)
%
%Generates a 2D difference of Gaussian (DOG) based on the following
%parameters:
%
%pWorld.n([1,2]))       size of image in pixels
%pWorld.visAngle        width of image in degrees of visual angle
%
%pRF.center             [x,y] center of DOG
%pRF.sig                1x2 vector of standard deviations of the two Gaussians
%pRF.contrast           1x2 vector of two amplitudes

%Written October 2008 by G.M. Boynton at the University of Washington

Gauss1 = pRF.contrast(1)*exp(-((pWorld.x(:,:,1)-pRF.center(1)).^2+(pWorld.y(:,:,1)-pRF.center(2)).^2)/pRF.sig(1)^2);
Gauss2 = pRF.contrast(2)*exp(-((pWorld.x(:,:,1)-pRF.center(1)).^2+(pWorld.y(:,:,1)-pRF.center(2)).^2)/pRF.sig(2)^2);
RF = Gauss1-Gauss2;

