%ContrastDiscriminationDemo.m

fontSize = 12;
%%  The basic Weber-Fechner relationship
% Contrast response functions predicting contrast discrimination thresholds.

%Define model parameters

pModel.p = 2.4;
pModel.q = 2.0;
pModel.sig = 0.1;
pModel.a = 1;

%Contrast values
pStim.contrast = exp(linspace(log(0.005),log(1),101));

R = nonlinearTransducer(pModel,pStim);  %the 'static nonlinearity'

figure(1)
clf
subplot(1,2,1)
plot((100*pStim.contrast),R);
set(gca,'Xlim',[min(100*pStim.contrast),max(100*pStim.contrast)]);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
set(gca,'FontSize',fontSize);
%Get Increment thresholds
pModel.inc = .1;

%% Get the thresholds and plot them.
thresh = binarySearch('nonlinearTransducer',pModel,pStim,'contrast');

subplot(1,2,2)
plot(log(100*pStim.contrast),log(100*thresh));
set(gca,'Xlim',[min(log(100*pStim.contrast)),max(log(100*pStim.contrast))]);
set(gca,'XTick',log([1,2,5,10,20,50,100]));
set(gca,'YTick',log([1,2,5,10,20,50,100]));
logx2raw(exp(1),0);
logy2raw(exp(1),0);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Increment threshold (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

%Graph the contrast increments and thresholds for a few example contrasts
cList = [.075,.5];

pStim.contrast = cList;
R = nonlinearTransducer(pModel,pStim);
thresh = binarySearch('nonlinearTransducer',pModel,pStim,'contrast');

for i=1:length(cList)
    subplot(1,2,1)
    hold on
    xlim = get(gca,'XLim');
    h= patch(100*[xlim(1)/100,cList(i),cList(i),cList(i)+thresh(i),cList(i)+thresh(i),xlim(1)/100],...
        [R(i),R(i),0,0,R(i)+pModel.inc,R(i)+pModel.inc],[.8,.8,1]);
    %set(h,'FaceAlpha',.5);
end



subplot(1,2,2)
hold on
plot(log(100*cList),log(100*thresh),'ko','MarkerFaceColor',[.8,.8,1]);

%% Predicting increment thresholds from the Heeger normalization model

% Set up 'world' parameters

%set up 'world' parameters
pWorld.n = [200,200,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 2;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% Model Parameters

clear pModel
%Half wave rectification and nonlinearity parameters
pModel.k = 500;         %scale factor
pModel.Vrest = 0;       %threshold
pModel.p = 2;           %exponent

%Normalization parameters
pModel.sig = .15;        %semisaturation constant
pModel.a = 1;           %scale factor

%% Create a population of spatio-temporal frequency filters
clear pRF RF

sfList =[1,2,3];    %list of preferred spatial frequencies
ctList = ones(size(sfList)); %[1,1,1,1,1];           %list of scale factors for these sf filters
angList = 0:45:179;         %list of preferred orientations
phaseList = [0,90,180,270]; %Quadrature phases

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf = NaN;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = NaN;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = NaN;              %orientation (deg) (to be varied)
pRF.phase = 0;              %0 = sin phase
pRF.contrast = NaN;           %scale factor (to be varied with sf)
pRF.nt = 1;                 %number of temporal frames

%Generate a cell array containing the receptive fields by calling 'makeGaborRF'
%Columns (1st dimension) will be orientation
%Rows (2nd dimension0 will be spatial frequency
%Third dimension are the four phases (Quadrature phase)
for i=1:length(sfList)
    pRF.sf = sfList(i);         %spatial frequency
    pRF.sig = .75/pRF.sf;       %width is inversely proportional to sf
    pRF.contrast = ctList(i);   %contrast
    for j=1:length(angList)
        pRF.ang = angList(j);   %orientation
        for k=1:4  %quadrature phases
            pRF.phase = phaseList(k);
            RF{i,j,k} = makeGaborRF(pWorld,pRF);
        end
    end
end

%Determine which cell to use for plotting
ci = find(sfList == 2);         %Spatial frequency index of the cell we'll be plotting (1 c/deg)
cj = find(angList == 90);       %Orientation index of the cell we'll be plotting (horizontal)

%% Contrast response function for mask and target+mask

%sfList = [1,2,3];
sfList = linspace(1,3,11);

pModel.inc = 1;
ctList = [0,exp(linspace(log(0.005),log(1),21))];
thresh = zeros(length(sfList),length(ctList));
for sfNum = 1:length(sfList);


    %Define the masking grating stimulus
    pMaskStim.type = 'grating';
    pMaskStim.sf = sfList(sfNum);                   %spatial frequency (c/deg)
    pMaskStim.ang = 90;                     %orientation (90 is horizontal)
    pMaskStim.phase = 0;                   %spatial phase
    pMaskStim.contrast = ctList;            %contrast [0-1]
    pMaskStim.p0 = [0,0];                   %center

    %Define the target grating stimulus
    pTargetStim.type = 'grating';
    pTargetStim.sf = 2;                   %spatial frequency (c/deg)
    pTargetStim.ang = 90;                     %orientation (90 is horizontal)
    pTargetStim.phase = 0;
    %    pTargetStim.sig = .5;
    pTargetStim.contrast = NaN;            %contrast [0-1]
    pTargetStim.p0 = [0,0];                   %center

    %Calculate the base response
    clear maskStim
    maskResp = zeros(size(ctList));
    for i=1:length(ctList)
        pMaskStim.contrast = ctList(i);
        maskStim{i} = makeStim(pWorld,pMaskStim);  %horizontally oriented grating
        C = normalizationModel(pWorld,pModel,maskStim{i},RF,'index',[ci,cj]);
        maskResp(i) = C;
    end

    low = zeros(size(ctList));
    high = ones(size(ctList));

    nIter = 11;

    for i=1:nIter
        mid = (high+low)/2;

        maskAndTargetResp = zeros(size(ctList));
        for j=1:length(ctList)
            pTargetStim.contrast = mid(j);
            targetStim = makeStim(pWorld,pTargetStim);  %horizontally oriented grating
            C = normalizationModel(pWorld,pModel,maskStim{j}+targetStim,RF,'index',[ci,cj]);
            maskAndTargetResp(j) = C;
        end

        deltaR = (maskAndTargetResp-maskResp);

        high(deltaR>pModel.inc) = mid(deltaR>pModel.inc);
        low(deltaR<pModel.inc) = mid(deltaR<pModel.inc);
    end
    high(high==1) = NaN;
    thresh(sfNum,:) = (high+low)/2;

end

figure(2)
clf
plot(log(100*ctList),log(100*thresh'));
set(gca,'Xlim',[min(log(100*ctList)),max(log(100*ctList))]);
set(gca,'XTick',log([1,2,5,10,20,50,100]));
set(gca,'YTick',log([1,2,5,10,20,50,100]));
logx2raw(exp(1),0);
logy2raw(exp(1),0);
xlabel('Contrast (%)','FontSize',fontSize);
ylabel('Increment threshold (%)','FontSize',fontSize);
set(gca,'FontSize',fontSize);

legend(num2str(sfList'));

%% 'Masking' function

figure(3)
clf

surf(log(100*ctList),sfList,log(100*thresh));
logx2raw(exp(1),0);

ylabel('Masker spatial frequency (c/deg)','FontSize',fontSize);
xlabel('Masker contrast (c/deg)','FontSize',fontSize);
zlabel('Increment threshold','FontSize',fontSize);
set(gca,'Xlim',[min(log(100*ctList)),max(log(100*ctList))]);
title(sprintf('Target %3.0f c/deg',pTargetStim.sf),'FontSize',fontSize);
set(gca,'FontSize',fontSize);






%% Make stimuli for powerpoint slides

clear maskStim
pMaskStim.type = 'grating';
pMaskStim.sf = 4;                   %spatial frequency (c/deg)
pMaskStim.ang = 90;                     %orientation (90 is horizontal)
pMaskStim.phase = 0;                   %spatial phase
pMaskStim.contrast = .1;            %contrast [0-1]
pMaskStim.p0 = [0,0];                   %center

%Define the target grating stimulus
pTargetStim.type = 'Gabor';
pTargetStim.sf = 2;                   %spatial frequency (c/deg)
pTargetStim.ang = 90;                     %orientation (90 is horizontal)
pTargetStim.phase = 0;                   %spatial phase
pTargetStim.contrast = .25;            %contrast [0-1]
pTargetStim.sig =.5;
pTargetStim.p0 = [0,0];                   %center

maskStim = makeStim(pWorld,pMaskStim);  %horizontally oriented grating
targetStim = makeStim(pWorld,pTargetStim);  %horizontally oriented grating


figure(4)
clf
image((maskStim+1)*127);
colormap(gray(256))
axis equal
axis off
set(gcf,'PaperPosition',[1,1,3,3]);

figure(5)
clf
image((targetStim+1)*127);
colormap(gray(256))
axis equal
axis off
set(gcf,'PaperPosition',[1,1,3,3]);

figure(6)
clf
image((targetStim+maskStim+1)*127);
colormap(gray(256))
axis equal
axis off
set(gcf,'PaperPosition',[1,1,3,3]);



