%FourierStuff.m

%% Set up 'world' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [400,400,1];      %[ny, nx, nt] in pixels
pWorld.visAngle = 4;        %horizontal extent of visual angle
pWorld.dur = 0;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% Start with 1D FFT 

nt  =101;
t = linspace(0,1,nt);
t = t(1:end-1);

freqs = 1*[1:2:31];
amps = sqrt(2/pi)./freqs;

y = zeros(size(t));
for i=1:length(freqs)
    y = y+amps(i)*sin(2*pi*freqs(i)*t);
end



figure(1)
clf
subplot(1,2,1)
plot(t,y);
xlabel('Time (sec)','FontSize',fontSize);
set(gca,'YLim',[-1.1,1.1]);
f = fft(y);

[dc,amp,ph] = myfft(y);

maxFreq = 15;

allFreqs = [0:length(amp)]/max(t);

id = allFreqs<maxFreq;
subplot(1,2,2)
stem(allFreqs(id),[dc,amp(id(2:end))],'MarkerFaceColor','b');
xlabel('Frequency (Hz)');
ylabel('Amplitude');
set(gca,'YLim',[-.05,1.1]);
set(gca,'XLim',[-.5,maxFreq+.5]);

set(gcf,'PaperPosition',[1,1,4,2]);

%% 1D FFT's of gaussians and gabors

nt  =101;
t = linspace(0,1,nt);
t = t(1:end-1);


sig =.1;
Gauss = exp(-(t-.5).^2/sig^2);

freq = 5;


y = Gauss.*cos(2*pi*(t-.5)*freq);
hold on


figure(1)
clf
subplot(1,2,1)
plot(t,y);
hold on
plot(t,Gauss,'r:');
plot(t,-Gauss,'r:');
xlabel('Time (sec)','FontSize',fontSize);
set(gca,'YLim',[-1.1,1.1]);
f = fft(y);

[dc,amp,ph] = myfft(y);
dc =dc*2;
maxFreq = 15;

allFreqs = [0:length(amp)]/max(t);

allFreqs= [-fliplr(allFreqs(2:end)),allFreqs];
y = [fliplr(amp),dc,amp];

id = abs(allFreqs)<maxFreq;
subplot(1,2,2)
plot(allFreqs(id),y(id),'b-');
xlabel('Frequency (Hz)');
ylabel('Amplitude');
set(gca,'YLim',[-.05,1.1]);
set(gca,'XLim',[-maxFreq-.5,maxFreq+.5]);

set(gcf,'PaperPosition',[1,1,4,2]);




%% 2D FFT of grating stimuli


%Define the grating stimulus
pStim.type = 'grating';
pStim.sf = 1;                       %spatial frequency (c/deg)
pStim.ang = 45;                     %orientation (90 is horizontal)
pStim.phase = 90;                   %spatial phase
pStim.contrast = 1;               %contrast [0-1] (to be varied)
pStim.p0 = [0,0];                   %center

stim = makeStim(pWorld,pStim);  %horizontally oriented grating

figure(1)
clf
plotFFT(pWorld,stim,1);

set(gcf,'PaperPosition',[1,1,4,2]);


%% FFT of Gaussian


%Define the DOG parameters 
clear pRF RF
pRF.center = [0,0];
pRF.sig = [1,1];
pRF.contrast = [1,0];
RF = makeDOGRF(pWorld,pRF);

figure(2)
clf
plotFFT(pWorld,RF,5);
set(gcf,'PaperPosition',[1,1,4,2]);

%%  Spatial position

%Define the DOG parameters 
clear pRF RF
pRF.center = [-1,1];
pRF.sig = [1,1];
pRF.contrast = [1,0];
RF = makeDOGRF(pWorld,pRF);

figure(2)
clf
plotFFT(pWorld,RF,5);
set(gcf,'PaperPosition',[1,1,4,2]);

%% Difference of Gaussian

%Define the DOG parameters 
clear pRF RF
pRF.center = [0,0];
pRF.sig = [.25,1];
pRF.contrast = [1,.05];
RF = makeDOGRF(pWorld,pRF);


figure(2)
clf
plotFFT(pWorld,RF,5);
set(gcf,'PaperPosition',[1,1,4,2]);

%% Gabor

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf =  1;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = .25;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = 0;              %orientation (deg) (to be varied)
pRF.phase = -90;              %0 = sin phase
pRF.contrast = 1;           %scale factor (to be varied with sf)



RF = makeGaborRF(pWorld,pRF);

figure(3)
plotFFT(pWorld,RF,5);
set(gcf,'PaperPosition',[1,1,4,2]);

%% Spatial and Temporal frequency

%spatial parameters for Gabor receptive field
pRF.center = [0,0];         %[0,0] is center of image
pRF.sf =  1;               %'carrier' spatial frequency (c/deg) (to be varied)
pRF.sig = .25;              %Gaussian standard deviation (to be varied with sf)
pRF.ang = 0;              %orientation (deg) (to be varied)
pRF.phase = -90;              %0 = sin phase
pRF.contrast = 1;           %scale factor (to be varied with sf)
pRF.tf = 1;





RF = makeGaborRF(pWorld,pRF);

figure(3)
plotFFT(pWorld,RF,5);
set(gcf,'PaperPosition',[1,1,4,2]);


