%LinearNonlinearModelDemo.m
%
%Demonstrates some of the propertis of the LN model
%
%Written in October 2008 by G.M. Boynton at the University of Washington

%% 'World' parameters
fontSize =12;

%set up 'world' parameters
pWorld.n = [50,50,1001];   %[ny, nx, nt] in pixels
pWorld.visAngle = 10;       %horizontal extent of visual angle
pWorld.dur = 1;             %total stimulus duration (seconds)
pWorld.background = 0;      %background color: [-1,1]

pWorld = initWorld(pWorld); %get the x and y meshrid matrices

%% Linear RF in Space and Time

clear pRF

%spatial parameters
pRF.center = [0,0];
pRF.sf = .25;  %c/deg
pRF.sig = 1.75; %
pRF.ang = 90; %orientation (deg)
pRF.phase = 270; %0 = sin phase
pRF.contrast = 1;
%temporal parameters
pRF.n = [3,5]; %n cascades
pRF.theta = [.005,.005]; %sec
pRF.a = [.05,.05];
pRF.nt = 100; % frames

RF = makeSeparableSTFilter(pWorld,pRF);

%show the whole receptive field (as a movie)
figure(1)
showRF(pWorld,RF)

%show the space-time plot by slicing through one plane in space
nn = ceil(pWorld.n(2)/2);  %half the size of the second (x) dimension
figure(2)
showRF(pWorld,squeeze(RF(:,nn,:)));

%% Stimulus of simple impulses

stim = zeros(pWorld.n);

%some positive and negative pulses in and outside the center of the RF
stim((nn-5:nn+5),(nn-5:nn+5),200:210) = 1;
stim((nn-15:nn-5),(nn-5:nn+5),230:240) = -1;
stim((nn-15:nn-5),(nn-5:nn+5),400:410) = -1;
stim((nn-5:nn+5),(nn-5:nn+5),400:410) = 1;
stim(:,:,600:610) = repmat(sign(rand(pWorld.n(1),pWorld.n(2))-.5),[1,1,11]);

%Show the stimulus as a movie
figure(3)
showStimulus(pWorld,stim);

%% Linear response to the stimulus

Vm = linearResponse(pWorld,stim,RF);

figure(4)
clf
plot(pWorld.t,Vm);
xlabel('Time (s)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
set(gcf,'PaperPosition',[1,1,10,2]);
ylim([-.15,.15]);

%% Random stimulus, linear response, nonlinear response and spikes

stim = zeros(pWorld.n);
pStim.frame = 20;  %number of frames (msec) for each image
for i=1:pStim.frame:(pWorld.n(3)-pStim.frame)
    %img = 2*(rand(pWorld.n(1:2))-.5);
    %img = randn(pWorld.n(1:2));
    img = sign(rand(pWorld.n(1:2))-.5);
    stim(:,:,i:(i+pStim.frame-1))=repmat(img,[1,1,pStim.frame]);
end

%Linear response to stimulus
Vm = linearResponse(pWorld,stim,RF);

figure(5)
clf

%plot the linear response
subplot(3,1,1)
plot(pWorld.t,Vm);
xlabel('Time (s)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
set(gcf,'PaperPosition',[1,1,10,8]);

%power law model parameters

clear pModel
pModel.k = 10000000;  %scale fac tor
pModel.Vrest = 0;     %threshold
pModel.p = 3;         %exponent

spikeRate = powerLawModel(pModel,Vm);

%plot the average spike rate over time
subplot(3,1,2)
plot(pWorld.t,spikeRate);
xlabel('Time (s)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
set(gcf,'PaperPosition',[1,1,10,8]);

%convert spike rate to an individual trial of spikes and plot it.
spikes = spikeRate2Spikes(pWorld,spikeRate);
subplot(3,1,3)
stairs(pWorld.t,spikes);
xlabel('Time (s)','FontSize',fontSize);
ylabel('Response','FontSize',fontSize);
set(gcf,'PaperPosition',[1,1,10,8]);







