function thresh = getDiscriminationThreshold(pNeurons,pStim,varName,e)
%thresh = getDiscriminationThreshold(pNeurons,pStim,varName,e)
%
%Estimates the increment threshold based on an ideal observer analyis of
%model V1 responses.  

if ~exist('e','var')  %efficiency for pooling d-prime across neurons
    e = 1;
end

nNeurons = length(pNeurons);
baseVals = getfield(pStim,varName);

%Calculate response to base stimulus for all neurons
pBaseStim = pStim;
baseResp = zeros(nNeurons,length(baseVals));
baseVarResp =zeros(nNeurons,length(baseVals));
for i=1:nNeurons
    [baseResp(i,:),baseVarResp(i,:)] = getV1Response(pNeurons(i),pBaseStim);
end

%first, find lower and upper bounds for increments that bracked 0<dprime<1
low = zeros(size(baseVals));
high = .05*ones(size(baseVals));
nIter =21;

incResp = zeros(nNeurons,length(baseVals));
incVarResp =zeros(nNeurons,length(baseVals));
for i=1:nIter
    %Calculate dprime for the higher bound for increments
    pIncStim = setfield(pStim,varName,baseVals+high);
    for j=1:nNeurons
        [incResp(j,:),incVarResp(j,:)] = getV1Response(pNeurons(j),pIncStim);
    end
    dprime = abs(baseResp-incResp)./sqrt(.5*(baseVarResp+incVarResp));
    %pool the dprimes across the neurons to get the population dprime
    dprimePop = sqrt(e*sum(dprime.^2,1));
    %double the high values that aren't high enough
    high(dprimePop<1) = high(dprimePop<1)*2;
end

%Use a binary search algorithm to find increments that make dprime=1.
for i=1:nIter  
    %Choose the midpoint between the lower rand upper bounds for increments
    mid = (low+high)/2;
    %Calculate dprime for this midpoint (just like we did above for 'high')
    pIncStim = setfield(pStim,varName,baseVals+mid);
    for j=1:nNeurons
        [incResp(j,:),incVarResp(j,:)] = getV1Response(pNeurons(j),pIncStim);
    end
    dprime = abs(baseResp-incResp)./sqrt(.5*(baseVarResp+incVarResp));
    %pool the dprimes across the neurons to get the population dprime
    dprimePop = sqrt(e*sum(dprime.^2,1));

    %lower the upper bound when dPrimePop>1 (increment too large)
    high(dprimePop>1) = mid(dprimePop>1);
    %raise the lower bound when dPrimePop<1 (increment too small)
    low(dprimePop<1) = mid(dprimePop<1);
end
%Final estimate of increment threshold
thresh = (low+high)/2;

%Set values to NaN where the increment threshold could not be obtained.
thresh(dprimePop<.9999) = NaN;
if strcmp(varName,'c')
    thresh(thresh>1) = NaN;
end


