function RF = makeSeparableSTFilter(pWorld,pRF);
%RF = makeSeparableSTFilter(pWorld,pRF)
%
%Creates a 3D receptive field (filter) that is a separable function (product) 
%of a Gabor in space and a difference of Gamma functions in time.  
%
%Parameters:
%   pWorld.t            Time vector (see initWorld.m)
%   pWorld.n            1x2 vector containing image size
%   
%   pRF.center          Center of RF (0,0) is middle of image
%   pRF.sf              Spatial frequency of Gabor
%   pRF.sig             Standard deviation of Gabor
%   pRF.ang             Orientation of Gabor (0 is vertical)
%   pRF.phase           Spatial phase of Gabor (0) is sine phase ('edge')
%   pRF.contrast        Amplitude of Gabor (max of 1)
%   pRF.n               1x2 vector contaning # of cascades for the positive
%                       (1st) and negative (2nd) Gamma function
%   pRF.theta           1x2 vector containing time constants for two Gammas
%   pRF.a               1x2 vector containing the amplitudes of two Gammas
%   pRF.nt              Number of time frames for the RF
%
%SEE ALSO:  Gamma.m  makeGaborRF.m

%10/7/08 Written by G.M. Boynton at the University of Washington

t = pWorld.t(1:(min(pRF.nt,length(pWorld.t))));

g1 = Gamma(pRF.n(1),pRF.theta(1),t);
g2 = Gamma(pRF.n(2),pRF.theta(2),t);

g = pRF.a(1)*g1 - pRF.a(2)*g2;

RF = makeGaborRF(pWorld,pRF);
RF = RF(:)*g;
RF = reshape(RF,[pWorld.n(1),pWorld.n(2),pRF.nt]);