function stim= makeStim(pWorld,pStim)


switch(pStim.type)
    case 'bar'  %moving bar stimulus
        stim = pWorld.background*ones(pWorld.n);

        rampy = cos(pStim.ang*pi/180)*(pWorld.x-pStim.p0(1)) + (sin(pStim.ang*pi/180)*pWorld.y-pStim.p0(2));
        rampx = sin(pStim.ang*pi/180)*(pWorld.x-pStim.p0(1)) - (cos(pStim.ang*pi/180)*pWorld.y-pStim.p0(2));

        dist = pWorld.t*pStim.v;
        x = cos((pStim.dir-pStim.ang)*pi/180)*dist;
        y = sin((pStim.dir-pStim.ang)*pi/180)*dist;
        for i=1:pWorld.n(3)
            tmp =         rampx<=x(i)+pStim.length/2 & ...
                rampx>=x(i)-pStim.length/2 & ...
                rampy<=y(i)+pStim.width/2 & ...
                rampy>=y(i)-pStim.width/2;

            stim(:,:,i) =  tmp*(pStim.contrast-pWorld.background)+pWorld.background;
        end
    case 'spot'  %single spot of light
        stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));
        stim = pStim.contrast*(pWorld.x-pStim.p0(1)).^2 + (pWorld.y-pStim.p0(2)).^2 <= pStim.width^2;
    case 'grating'
        stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));
        ramp = cos(pStim.ang*pi/180)*(pWorld.x-pStim.p0(1)) + (sin(pStim.ang*pi/180)*pWorld.y-pStim.p0(2));
        stim = pStim.contrast*sin(2*pi*pStim.sf*ramp-pStim.phase);
    case 'Gabor'
        stim = pWorld.background*ones(pWorld.n(1),pWorld.n(2));
        ramp = cos(pStim.ang*pi/180)*(pWorld.x-pStim.p0(1)) + (sin(pStim.ang*pi/180)*pWorld.y-pStim.p0(2));
        gauss = exp(-((pWorld.x-pStim.p0(1)).^2+(pWorld.y-pStim.p0(1)).^2)/pStim.sig^2);
        stim = pStim.contrast*gauss.*sin(2*pi*pStim.sf*ramp-pStim.phase);
    case 'DriftingGrating'
        stim = pWorld.background*ones(pWorld.n);
        phList = pStim.phase + 2*pi*pStim.tf*pWorld.t;
        ramp = cos(pStim.ang*pi/180)*(pWorld.x-pStim.p0(1)) + (sin(pStim.ang*pi/180)*pWorld.y-pStim.p0(2));
        for i=1:length(phList)
            stim(:,:,i) = pStim.contrast*sin(2*pi*pStim.sf*ramp-phList(i));
        end
end





