function pNeuron = sampleV1Neurons(nNeurons)
%pNeuron = sampleV1Neurons(nNeurons)


%Geisler and Albrecht parameters


% Mean values

%baseline, variance and max response

pNeuronMean.r0 = 0.52;
pNeuronMean.K = 1.7;
pNeuronMean.rMax = 8.2;

%contrast
pNeuronMean.c50 = 0.43;
pNeuronMean.n = 2.5;

%spatial frequency
pNeuronMean.uc = 4.5;
pNeuronMean.hu = 0.8;
pNeuronMean.lu = 0.73;

%orientation
%pNeuronMean.thetac = 90;
pNeuronMean.btheta = 20;


%% Standard Deviation values

%baseline, variance and max response
pNeuronSD.r0 = 0.83;
pNeuronSD.K = .8;
pNeuronSD.rMax = 7.0;

%contrast
pNeuronSD.c50 = 0.31;
pNeuronSD.n = 1.4;

%sptial frequency
pNeuronSD.uc = 2.3;
pNeuronSD.hu = 0.29;
pNeuronSD.lu = 0.32;

%orientation
%pNeuronSD.thetac = inf;
pNeuronSD.btheta = 9.1;

%% Median values

%baseline, variance and max response
pNeuronMedian.r0 = 0.17;
pNeuronMedian.K = 1.3;
pNeuronMedian.rMax = 5.7;

%contrast
pNeuronMedian.c50 = 0.32;
pNeuronMedian.n = 2.0;

%sptial frequency
pNeuronMedian.uc = 4.2;
pNeuronMedian.hu = 0.78;
pNeuronMedian.lu = 0.66;

%orientation
%pNeuronMedian.thetac = inf;
pNeuronMedian.btheta = 15.0;

%% Skew normals to match these parameters


clear pNeuron
%initialize pNeuron fields
for i=1:nNeurons
    pNeuron(i).K = NaN;      %fanofactor (scale factor determining variance of response)
    pNeuron(i).r0 = NaN;       %baseline firing rate (spikes/sec)
    %contrast
    pNeuron(i).c50 = NaN;     %semisaturation constant
    pNeuron(i).n = NaN;       %exponent
    pNeuron(i).rMax = NaN;    %max response (spikes/sec)

    %spatial frequency
    pNeuron(i).uc = NaN;       %preferred spatial frequency (c/deg)
    pNeuron(i).hu = NaN;       %spatial frequency bandwithd (high)
    pNeuron(i).lu = NaN;       %spatial frequency bandwith (low)

    %orientation
    pNeuron(i).thetac = rand(1)*180;   %preferred orientation 0 = vertical
    pNeuron(i).btheta = NaN;   %orientation bandwidth (degrees)






end

fields = fieldnames(pNeuronMean);


for i=1:length(fields)


    varName = fields{i};
    if ~strcmp(varName,'thetac')
        m = getfield(pNeuronMean,varName);
        s = getfield(pNeuronSD,varName);
        med = getfield(pNeuronMedian,varName);

        x = randn(1,nNeurons).^2;

        params.exp = 1;
        params.sc = 1;

        params.shutup = 1;

        [params,err] = fit('skewedDistErr',params,{'exp','sc'},x,m,s,med);
        [err,xx] = skewedDistErr(params,x,m,s,med);


        for j=1:nNeurons
            pNeuron(j) = setfield(pNeuron(j),varName,xx(j));
        end
%         disp(sprintf('%s',varName));
%         disp(sprintf('mean:   %5.2f vs %5.2f',mean(xx),m));
%         disp(sprintf('median: %5.2f vs %5.2f',median(xx),med));
%         disp(sprintf('std:    %5.2f vs %5.2f',std(xx),s));
%         disp(' ')
    end

end






